#!/bin/bash

#############################################################################
# In this script, you will affect values to your tool in order to let
# the oracle launch it in an apropriate way.
#############################################################################

# BASE_DIR:		 is the base directory of your submission
# BIN_DIR:			is the directory containing the binaries
# MODELS_DIR:	 is the directory containing the directories for all models
# MODEL_DIR:		is the directory containing the current model
# MODEL:				is the name of the model that is being processed
# SCALE:				is the current scaling value
#							 (e.g., number of philosopher for the philosopher model)
#							 for the model that is being processed
# FORMULA_FILE: when evaluating a formula, this contains the path of the
#							 file that contains the formula to be processed

# The first parameter of this script is one of the cases described below.
#
# "-init":			Exports user-defined variables.
#							 You can initialize here your own variables,
#							 which are then available when executing the oracle
#							 as well as your tool.
#
# "-start":		 Pre-processing actions before processing each model
#							 (not each model instance).
#							 Typically, if you need some compilation phase
#							 (e.g. your tool is compiled for a given model),
#							 your may set up this action there.
#
# "-generate":	Lauch the model checker for a model instance to generate
#							 the state space
#
# "-deadlocks": Lauch the model checker for a model instance to compute
#							 the list of deadlocks
#
# "-check":		 Launch the model checker for a model instance to check for
#							 a formula
#
# "-stop":			Post-processing actions after each model
#							 (not each model instance).
#
# The cases "-start" and "-stop" are useful if your model checker runs
# in server mode. Feel free to start it in "-start", and stop it in "-stop".

case "$1" in
	"-init")
		# Fill these variables
		export TOOL_NAME="" # Please fill
		export TEAM_NAME="" # Please fill
		export INSTITUTION_NAME="" # Please fill
		export COUNTRY_NAME="" # Please fill
		export CONTACT_EMAIL="" # Please fill
		export CONTACT_NAME="" # Please fill
		export CONTACT_PHONE="" # Please fill
		export CONTACT_URL="" # Please fill
		# Add here your own code for global initialization (executed once before
		# processing the first model)
		;;
	"-start")
		# MODEL contains the model name
		# Add here your own code. Typically, if your tool needs to be compiled
		# to include the model description in its code, you may proced here.
		
		# Typically, if you have a parameterized version of a given model and if you
		# want to generate all the instances suggested in the variable SCALES for the
		# corresponding model, then you must perform this here.
		
		sh "$BIN_DIR/preparation-script.sh" # for the dunmmy tool provided as an example
		source "$MODEL_DIR/setup.sh" "-start"
		;;
	"-stop")
		# MODEL contains the model name
		source "$MODEL_DIR/setup.sh" "-stop"
		# Add here your own code.
		sh "$BIN_DIR/post-execution-script.sh" # for the dunmmy tool provided as an example
		;;
	"-generate")
		# MODEL contains the model name
		# SCALE contains the instance parameter
		source "$MODEL_DIR/setup.sh" "-generate"
		# Add here your own code.
		# IMPORTANT: the tool must generate the full state space and provides
		#						the number of explicit states in the system in stdout
		#						using the following format in two separated lines:
		#									STATE_SPACE <SVAL> TECHNIQUES <TVAL1> <TVAL2> ...
		#						where:
		#						- <SVAL> is the number of computed states,
		#						- <TVALx> are techniques selected by the tool to perform
		#							this operation (several may be used together).
		#							Classification of these techniques are done by means of
		#							the following keywords:
		#								- EXPLICIT
		#								- ABSTRACTIONS
		#								- PARTIAL_ORDERS
		#								- PARALLEL_PROCESSING
		#								- DECISION_DIAGRAMS
		#								- SYMMETRIES
		#								- STATE_COMPRESSION
		#								- OTHERS
		#						These result must be provided ONLY WHEN computation is
		#						completed, otherwise output must be:
		#						- CANNOT_COMPUTE if you detect that your tool cannot
		#							process the query
		#						- DO_NOT_COMPETE if you are in a class of model or request
		#							that your tool is not able to process.
		#						No other information must be provided in stdout.
		#
		# For the dummy tool, for instance:
		"$BIN_DIR/MY_TOOL" -state_space "$MODEL_DIR/desc_model-$SCALE" "$SCALE" # for the dunmmy tool provided as an example
		;;
	"-deadlocks")
		# MODEL contains the model name
		# SCALE contains the instance parameters
		source "$MODEL_DIR/setup.sh" "-deadlocks"
		# Add here your own code.
		# IMPORTANT: the tool must search for deadlocks and display the result
		#						in stdout using the following format:
		#									DEADLOCKS <DVAL> TECHNIQUES <TVAL1> <TVAL2> ...
		#						where:
		#						- <DVAL> is the number of deadlocks in the system,
		#						- <TVALx> are techniques selected by the tool to perform
		#							this operation (several may be used together).
		#							Classification of these techniques are done by means of
		#							the following keywords:
		#								- EXPLICIT
		#								- ABSTRACTIONS
		#								- PARTIAL_ORDERS
		#								- PARALLEL_PROCESSING
		#								- DECISION_DIAGRAMS
		#								- SYMMETRIES
		#								- STATE_COMPRESSION
		#								- OTHERS
		#						These result must be provided ONLY WHEN computation is
		#						completed, otherwise output must be:
		#						- CANNOT_COMPUTE if you detect that your tool cannot
		#							process the query
		#						- DO_NOT_COMPETE if you are in a class of model or request
		#							that your tool is not able to process.
		#						No other information must be provided in stdout.
		#
		# For the dummy tool, for instance:
		"$BIN_DIR/MY_TOOL" -dead "$MODEL_DIR/desc_model-$SCALE" "$SCALE" # for the dunmmy tool provided as an example
		;;
	"-check")
		# MODEL contains the model name
		# SCALE contains the instance parameters
		# FORMULA_FILE is the file containing the formulas to check
		source "$MODEL_DIR/setup.sh" "-check"
		# Add here your own code.
		# IMPORTANT: the tool must evaluate the formulas found in the file
		#						referenceed in FORMULA_FILE and following the syntax
		#						presented in the submission manual.
		#						When calculus is completed, it displays the result in stdout
		#						using the following format:
		#									 FORMULA <NAME1> <RES1> TECHNIQUES <TVAL1> <TVAL2>...
		#									 FORMULA <NAME2> <RES1> TECHNIQUES <TVAL1> <TVAL2>...
		#						where:
		#						- <NAME> is the name of the formula in the file,
		#						- <RES> is TRUE or FALSE,
		#						- <TVALx> are techniques selected by the tool to perform
		#							this operation (several may be used together).
		#							Classification of these techniques are done by means of
		#							the following keywords:
		#								- EXPLICIT
		#								- ABSTRACTIONS
		#								- PARTIAL_ORDERS
		#								- PARALLEL_PROCESSING
		#								- DECISION_DIAGRAMS
		#								- SYMMETRIES
		#								- STATE_COMPRESSION
		#								- OTHERS
		#						These result must be provided ONLY WHEN computation is
		#						completed, otherwise output must be:
		#						- CANNOT_COMPUTE if you detect that your tool cannot
		#							process the query
		#						- DO_NOT_COMPETE if you are in a class of model or request
		#							that your tool is not able to process.
		#						No other information must be provided in stdout.
		#
		# For the dummy tool, for instance:
		"$BIN_DIR/MY_TOOL" -formula "$MODEL_DIR/desc_model-$SCALE" "$SCALE" "$FORMULA_FILE" # for the dunmmy tool provided as an example
		;;
	*)
		echo "$0: Wrong invocation:" >> $LOG_FILE
		exit 1
		;;
esac
